@@ -1,7 +1,7 @@
 /* This file is part of TCD.
    gtcd.c - Main source file for GTK+ interface.
    
-   Copyright (C) 1997-98 Tim P. Gerla <timg@means.net>
+   Copyright (C) 1997-98 Tim P. Gerla <timg@rrv.net>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
    Tim P. Gerla
    RR 1, Box 40
    Climax, MN  56523
-   timg@means.net 						
+   timg@rrv.net 						
 */
 
 #include <config.h>
@@ -95,7 +95,8 @@ GtkTooltips *tooltips;
 
 int timeonly = FALSE, status_height, status_width, playid=-1;
 int configured = FALSE, old_status=-1, max,tfont_height;
-unsigned int cur_goto_id, release_t=0, roll_t=0;
+unsigned long cur_goto_id=0;
+unsigned int release_t=0, roll_t=0;
 tcd_prefs *prefs;
 int cddb=0, max_title_width;
 int time_display;
@@ -113,7 +114,6 @@ gint release_timer(gpointer *data);
 int skip_cb(GtkWidget *widget, GdkEvent *event, gpointer *data);
 gint changer_callback(GtkWidget *widget, gpointer *data);
 GtkWidget* make_changer_buttons(void);
-GtkWidget* make_small_buttons(void);
 GtkWidget* create_buttons(void);
 void setup_colors(void);
 void draw_time_playing(void);
@@ -282,7 +282,7 @@ GtkWidget* make_changer_buttons( void )
 
 		style = gtk_widget_get_style(GTK_BUTTON(changer_buttons[i])->child);
 		gdk_font_unref(style->font);
-		style->font = gdk_font_load("-adobe-helvetica-medium-r-normal-*-*-100-*-*-p-*-iso8859-1");
+		style->font = gdk_fontset_load(_("-adobe-helvetica-medium-r-normal-*-*-100-*-*-p-*-*-*"));
 		gdk_font_ref(style->font);
 		gtk_widget_set_style(GTK_BUTTON(changer_buttons[i])->child, style);
 		gdk_font_unref(style->font);
@@ -310,24 +310,6 @@ static gint button_press (GtkWidget *widget, GdkEvent *event)
 	return FALSE;
 }
 
-GtkWidget *make_small_buttons(void)
-{
-	GtkWidget *b1, *b2, *b3;
-	GtkWidget *table;
-    
-	table = gtk_table_new(TRUE, 6,1);
-
-	b1 = make_button_stock(GNOME_STOCK_PIXMAP_PROPERTIES, edit_window, &prefs->tracked, _("Open track editor"));
-	b2 = make_button_stock(GNOME_STOCK_PIXMAP_PREFERENCES, preferences, NULL, _("Preferences"));
-	b3 = make_button_stock(GNOME_STOCK_PIXMAP_QUIT, quit_cb, &prefs->quit, _("Quit"));
-
-	gtk_table_attach_defaults(GTK_TABLE(table), b1, 0, 2, 0, 1);
-	gtk_table_attach_defaults(GTK_TABLE(table), b2, 2, 4, 0, 1);
-	gtk_table_attach_defaults(GTK_TABLE(table), b3, 4, 6, 0, 1);
-
-	return table;
-}
-
 GtkWidget* create_buttons(void)
 {
 	GtkWidget *table;
@@ -377,7 +359,14 @@ GtkWidget* create_buttons(void)
 	gtk_table_attach_defaults(GTK_TABLE(table), rw, 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(table), ff, 1, 2, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(table), gotobutton, 2, 3, 1, 2);
-	gtk_table_attach_defaults(GTK_TABLE(table), make_small_buttons(), 0, 3, 2, 3);
+
+	b1 = make_button_stock(GNOME_STOCK_PIXMAP_PROPERTIES, edit_window, &prefs->tracked, _("Open track editor"));
+	b2 = make_button_stock(GNOME_STOCK_PIXMAP_PREFERENCES, preferences, NULL, _("Preferences"));
+	b3 = make_button_stock(GNOME_STOCK_PIXMAP_QUIT, quit_cb, &prefs->quit, _("Quit"));
+
+	gtk_table_attach_defaults(GTK_TABLE(table), b1, 0, 1, 2, 3);
+	gtk_table_attach_defaults(GTK_TABLE(table), b2, 1, 2, 2, 3);
+	gtk_table_attach_defaults(GTK_TABLE(table), b3, 2, 3, 2, 3);
 
 	return table;
 }
@@ -443,7 +432,7 @@ void draw_time_playing(void)
 	}		
 }
 
-char *calculate_title(int maxwidth, int inc)
+void calculate_title(int maxwidth, int inc)
 {
 	int start = 0, end = 0;
 	int width, slen, w;
@@ -490,7 +479,8 @@ void draw_titles(void)
 {
 	int inc;
 
-	gdk_gc_set_foreground( gc, &track_color );
+	if(gc && &track_color)
+		gdk_gc_set_foreground( gc, &track_color );
 	inc = (tfont->ascent+tfont->descent)-2;
 
 	gdk_draw_text(status_db,tfont,gc,4,39, cd.artist,
@@ -691,11 +681,8 @@ gint volume_changed( GtkWidget *widget, gpointer *data )
 
 gint launch_gmix( GtkWidget *widget, GdkEvent *event, gpointer data ) 
 {
-	if ( (event->type==GDK_2BUTTON_PRESS) && (fork()==0) )
-	{
-		execlp("gmix","",NULL); 
-		_exit(0);
-	}
+	if (event->type==GDK_2BUTTON_PRESS)
+	  mixer_cb(NULL, NULL);
 
 	return FALSE;
 }
@@ -706,11 +693,13 @@ void make_goto_menu()
 	int i;
 	GtkWidget *item;
 
+	if(gotomenu)
+		gtk_widget_destroy(gotomenu);
 	gotomenu = gtk_menu_new();
 
 	for(i=1; i <= cd.last_t; i++)
 	{
-		g_snprintf(buf, TRK_NAME_LEN-1, "%2d - %s", i, cd.trk[C(i)].name);
+		g_snprintf(buf, TRK_NAME_LEN, "%2d - %s", i, cd.trk[C(i)].name);
 		item = gtk_menu_item_new_with_label(buf);
 		gtk_widget_show(item);
 		gtk_menu_append(GTK_MENU(gotomenu), item);
@@ -895,7 +884,7 @@ void setup_time_display(GtkWidget *table)
 	else
 		gtk_box_pack_start( GTK_BOX(status_table), frame, TRUE, TRUE, 0);
 		                
-	gtk_box_pack_start(GTK_BOX(main_box), status_table, FALSE, FALSE, 2);
+	gtk_box_pack_start(GTK_BOX(main_box), status_table, TRUE, TRUE, 0);
 	return;
 }
 
@@ -905,14 +894,14 @@ void setup_fonts(void)
 		gdk_font_unref(tfont);
 
 	if(prefs->trackfont)
-		tfont = gdk_font_load( prefs->trackfont );
+		tfont = gdk_fontset_load( prefs->trackfont );
 	else
 		tfont = NULL;
 
 	if(sfont)
 		gdk_font_unref(sfont);
     
-	sfont = gdk_font_load("-adobe-helvetica-medium-r-normal-*-*-100-*-*-p-*-iso8859-1");
+	sfont = gdk_fontset_load(_("-adobe-helvetica-medium-r-normal-*-*-100-*-*-p-*-*-*"));
     
 	tfont_height = (tfont->ascent+tfont->descent)*3;
 }
@@ -924,12 +913,12 @@ void init_window(void)
 	window = gnome_app_new("gtcd", "TCD 2.0");
 	gtk_window_set_title(GTK_WINDOW(window), PACKAGE" "VERSION" ");
 	gtk_window_set_wmclass( GTK_WINDOW(window), "main_window","gtcd");
-	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);
+	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, TRUE);
 
 	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
 			   GTK_SIGNAL_FUNC(quit_cb), NULL);
 
-	gtk_container_border_width(GTK_CONTAINER(window), GNOME_PAD_SMALL);
+	gtk_container_set_border_width(GTK_CONTAINER(window), GNOME_PAD_SMALL);
 
 	gtk_widget_realize(window);
 
@@ -1040,6 +1029,7 @@ int main (int argc, char *argv[])
 
 	cd.play_method = NORMAL;
 
+	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gtcd.png");
 	prefs = g_new0(tcd_prefs, 1);
 	load_prefs(prefs);
 	time_display = gnome_config_get_int("/gtcd/ui/time_display=0");
