@@ -26,7 +26,7 @@ static void edit_disc_extra(GtkWidget *widget);
 static void edit_track_extra(GtkWidget *widget, GtkWidget *list);
 static void gtracked_submit(void);
 
-static void destroy_window (GtkWidget *widget, gboolean save)
+static void destroy_window(GtkWidget *widget, gboolean save)
 {
     if(save)
 	tcd_writediskinfo(&cd);
@@ -68,28 +68,25 @@ static void update_list( GtkWidget *list, int track )
 static void fill_list( GtkWidget *list )
 {
     int i;
-    char *tmp[4];
+    char *tmp[3];
+    char one[TRK_NAME_LEN], two[TRK_NAME_LEN];
 
-    for(i=0; i < 4; i++)
-	tmp[i] = g_malloc(TRK_NAME_LEN);
+    tmp[0] = one;
+    tmp[1] = two;
 
     gtk_clist_freeze(GTK_CLIST(list));
 
     for( i=cd.first_t; i <= cd.last_t; i++ )
     {
-	g_snprintf(tmp[0], TRK_NAME_LEN-1, "%d", i);
-	g_snprintf(tmp[1], TRK_NAME_LEN-1, "%2d:%02d",
+	g_snprintf(tmp[0], TRK_NAME_LEN, "%d", i);
+	g_snprintf(tmp[1], TRK_NAME_LEN, "%2d:%02d",
 		   cd.trk[i].tot_min, cd.trk[i].tot_sec);
-	g_snprintf(tmp[2], TRK_NAME_LEN-1, "%s", cd.trk[i].name);
-	tmp[3] = NULL;
+	tmp[2] = cd.trk[i].name;
 	
 	gtk_clist_append(GTK_CLIST(list), tmp);
-	g_snprintf(tmp[0],255, "%d", i);
     }
 
     gtk_clist_thaw(GTK_CLIST(list));
-    for(i=0; i < 4; i++)
-	g_free(tmp[i]);
 		
     return;
 }
@@ -148,7 +145,11 @@ static void select_row_cb( GtkCList *clist,
     row++;
     gtk_object_set_user_data(GTK_OBJECT(clist), GINT_TO_POINTER(row));
 
-    gtk_entry_set_text(GTK_ENTRY(entry), cd.trk[row].name);
+    if(strncmp(cd.trk[row].name, "Track", 5))
+	    gtk_entry_set_text(GTK_ENTRY(entry), cd.trk[row].name);
+    else
+	    gtk_entry_set_text(GTK_ENTRY(entry), "");
+
     gtk_signal_handler_block_by_func(GTK_OBJECT(extra),
 				     GTK_SIGNAL_FUNC(edit_track_extra), clist);
     gtk_editable_delete_text(GTK_EDITABLE(extra), 0, -1);
@@ -189,7 +190,8 @@ void edit_window(GtkWidget *widget, gpointer data)
     }
     
     trwin = gtk_window_new(GTK_WINDOW_DIALOG);
-    gtk_container_border_width(GTK_CONTAINER(trwin), 5);
+    gnome_window_icon_set_from_default (GTK_WINDOW (trwin));
+    gtk_container_border_width(GTK_CONTAINER(trwin), GNOME_PAD_SMALL);
     gtk_window_set_title(GTK_WINDOW(trwin), _("Track Editor"));
     gtk_window_set_wmclass(GTK_WINDOW(trwin), "track_editor","gtcd");
     gtk_widget_set_usize(trwin, 500, 300);
@@ -325,8 +327,6 @@ void edit_window(GtkWidget *widget, gpointer data)
     gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		       GTK_SIGNAL_FUNC(destroy_window), (gpointer)FALSE);
 
-    gtk_signal_connect(GTK_OBJECT(track_list), "select_row",
-		       GTK_SIGNAL_FUNC(select_row_cb), trwin);
     gtk_signal_connect(GTK_OBJECT(disc_entry), "changed",
 		       GTK_SIGNAL_FUNC(dtitle_changed), NULL);
     signal_id = gtk_signal_connect(GTK_OBJECT(track_entry), "changed",
@@ -343,6 +343,9 @@ void edit_window(GtkWidget *widget, gpointer data)
 
     fill_list(track_list);
 
+    gtk_signal_connect(GTK_OBJECT(track_list), "select_row",
+		       GTK_SIGNAL_FUNC(select_row_cb), trwin);
+
     gtk_widget_show_all(trwin);
     return;
 }
