@@ -4,7 +4,7 @@
    All the functions that start with tcd_ are here, and aren't
    (shouldn't be...) dependent on any user interface.
 
-   Copyright (C) 1997-98 Tim P. Gerla <timg@means.net>
+   Copyright (C) 1997-98 Tim P. Gerla <timg@rrv.net>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
    Tim P. Gerla
    RR 1, Box 40
    Climax, MN  56523
-   timg@means.net
+   timg@rrv.net
 */
 
 #include <stdlib.h>
@@ -353,7 +353,8 @@ int tcd_playtracks(cd_struct *cd, int start_t, int end_t, int only_use_trkind)
     }
 
 #if defined(CDROMCLOSETRAY)
-    ioctl(cd->cd_dev, CDROMCLOSETRAY);
+    if( ioctl( cd->cd_dev, CDROM_DRIVE_STATUS ) == CDS_TRAY_OPEN )
+	    ioctl(cd->cd_dev, CDROMCLOSETRAY);
 #endif	        
 	
     if(cd->trk[start_t].toc.cdte_ctrl == CDROM_DATA_TRACK)
