@@ -1,7 +1,7 @@
 /* This file is part of TCD 2.0.
    cdrom.h - Interface independant CDROM routines.
 
-   Copyright (C) 1997-98 Tim P. Gerla <timg@means.net>
+   Copyright (C) 1997-98 Tim P. Gerla <timg@rrv.net>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
    Tim P. Gerla
    RR 1, Box 40
    Climax, MN  56523
-   timg@means.net
+   timg@rrv.net
 */
 #ifndef TCD_CDROM_H
 #define TCD_CDROM_H
@@ -28,14 +28,15 @@
 #include <sys/types.h>
 #include <glib.h>
 #if !defined(linux) && !defined(sun) && !defined(__sun__)
-#error TCD only builds on linux and Solaris/SunOs
+# error TCD only builds on linux and Solaris/SunOs
 #endif
 
 #ifdef linux
-#include <linux/cdrom.h>
-
-typedef struct cdrom_msf0 cd_min_sec_frame;
-
+# include <linux/cdrom.h>
+  typedef struct cdrom_msf0 cd_min_sec_frame;
+# ifdef HAVE_LINUX_UCDROM_H
+#  include <linux/ucdrom.h>
+# endif
 #endif
 
 #if defined(sun) || defined(__sun__)
