@@ -1,6 +1,6 @@
 /* This file is part of TCD 2.0.
    
-   Copyright (C) 1997-98 Tim P. Gerla <timg@means.net>
+   Copyright (C) 1997-98 Tim P. Gerla <timg@rrv.net>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
    Tim P. Gerla
    RR 1, Box 40
    Climax, MN  56523
-   timg@means.net
+   timg@rrv.net
 */
 
 #include <config.h>
@@ -60,6 +60,7 @@ void preferences(GtkWidget *widget, void *data);
 
 void load_prefs(tcd_prefs *prop)
 {
+	char *s;
 #if defined(sun) || defined(__sun__)
 #if defined(SVR4) || defined(__svr4__)
 	prop->cddev=gnome_config_get_string    ("/gtcd/cdrom/device=/vol/dev/aliases/cdrom0");
@@ -72,7 +73,10 @@ void load_prefs(tcd_prefs *prop)
 	prop->handle=gnome_config_get_bool     ("/gtcd/ui/handle=false");
 	prop->tooltip=gnome_config_get_bool    ("/gtcd/ui/tooltip=true");
 	prop->mixer_cmd=gnome_config_get_string    ("/gtcd/ui/mixer=gmix");
-	prop->trackfont=gnome_config_get_string("/gtcd/ui/trackfont=-adobe-helvetica-medium-r-normal-*-*-120-*-*-p-*-iso8859-1");
+
+	s = g_strconcat("/gtcd/ui/trackfont=", _("-adobe-helvetica-medium-r-normal-*-*-120-*-*-p-*-*-*"), NULL);
+	prop->trackfont=gnome_config_get_string(s);
+	g_free(s);
 
 	prop->trackcolor_r=gnome_config_get_int("/gtcd/ui/trackcolor_r=255" );
 	prop->trackcolor_g=gnome_config_get_int("/gtcd/ui/trackcolor_g=0" );
@@ -100,7 +104,9 @@ void load_prefs(tcd_prefs *prop)
 	prop->cddb_httpproxy_need_auth = gnome_config_get_bool("/cddbslave/server/need_http_proxy_auth=false");
 	prop->cddb_httpproxy_auth_name = gnome_config_private_get_string("/cddbslave/server/http_proxy_auth_name=");
 	prop->cddb_httpproxy_auth_passwd = gnome_config_private_get_string("/cddbslave/server/http_proxy_auth_passwd=");
-	prop->cddb_httpproxy = gnome_config_get_string("/cddbslave/server/http_proxy=");
+	prop->cddb_httpproxy = gnome_config_get_string("/cddbslave/server/http_proxy=proxy");
+	prop->use_socks = gnome_config_get_bool_with_default("/cddbslave/server/use_socks=false", NULL);
+	prop->socks_server = gnome_config_get_string("/cddbslave/server/socks_server=socks");
 }
 
 void save_prefs(tcd_prefs *prop)
@@ -137,6 +143,8 @@ void save_prefs(tcd_prefs *prop)
 	gnome_config_private_set_string("/cddbslave/server/http_proxy_auth_passwd", prop->cddb_httpproxy_auth_passwd);
 
 	gnome_config_set_bool("/gtcd/general/only_use_trkind", prop->only_use_trkind);
+	gnome_config_set_bool("/cddbslave/server/use_socks", prop->use_socks);
+	gnome_config_set_string("/cddbslave/server/socks_server", prop->socks_server);
         
 	gnome_config_sync();
 }
@@ -299,6 +307,7 @@ static void font_button_cb(GtkWidget *widget, gpointer *data)
         GtkWidget *fs;
         
         fs = gtk_font_selection_dialog_new("Font");
+	gnome_window_icon_set_from_default (GTK_WINDOW (fs));
         gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(fs), prefs->trackfont);
 
         gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(fs)->ok_button), "clicked",
