@@ -4,7 +4,7 @@
 /* This file is part of TCD 2.0.
    tcd.c - Main source file for curses interface.
    
-   Copyright (C) 1997-98 Tim P. Gerla <timg@means.net>
+   Copyright (C) 1997-98 Tim P. Gerla <timg@rrv.net>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
    Tim P. Gerla
    RR 1, Box 40
    Climax, MN  56523
-   timg@means.net
+   timg@rrv.net
 */
    
 
@@ -650,7 +650,7 @@ void sighandler(int sig)
     endwin();
 	
     printf( "Signal %d caught, exiting.\n", sig );
-    printf( "Please mail any info (no core dumps please.) to timg@means.net.\n\n" );
+    printf( "Please mail any info (no core dumps please.) to timg@rrv.net.\n\n" );
 	
     exit(sig);
 } 
